/*
    Bantumi
    Copyright 2005, 2006 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsj
    martin@martin.st
*/

#include <coemain.h>
#include "bantumiappview.h"
#include <flogger.h>
#include "bantumi.h"
#include "bantumigl.h"
#include "glfont.h"
#include <hal.h>
#include <eikenv.h>
#include <GLES/gl.h>
#include <aknutils.h>
#include "bantumiappui.h"

#include <time.h>
#include <stdlib.h>

GLFont *makeSymbianGLFont(CFbsBitGc *gc, int height);

void debug(const char *str) {
	TBuf<100> buf;
	while (*str != '\0')
		buf.Append(*str++);
	RFileLogger::WriteFormat(_L("debug"), _L("bantumi.txt"), EFileLoggingModeAppend, _L("%S"), &buf);
}

CBantumiAppView* CBantumiAppView::NewL(const TRect& aRect, TBool aFullscreen) {
	CBantumiAppView* self = CBantumiAppView::NewLC(aRect, aFullscreen);
	CleanupStack::Pop(self);
	return self;
}

CBantumiAppView* CBantumiAppView::NewLC(const TRect& aRect, TBool aFullscreen) {
	CBantumiAppView* self = new(ELeave) CBantumiAppView();
	CleanupStack::PushL(self);
	self->ConstructL(aRect, aFullscreen ? EFullscreen : EWindowed, EPixmap, ETrue, ETrue);
	return self;
}

void CBantumiAppView::ConstructAppL() {
	srand(time(NULL));
	iBantumiGL = new(ELeave) BantumiGL(Rect().Width(), Rect().Height());
	InitFonts();

	iBantumi = new(ELeave) Bantumi(iBantumiGL);
	iBantumiGL->useScissor(iDrawMode == EPixmap);
}

void CBantumiAppView::InitFonts() {
	delete iGLTextFont;
	delete iGLNumFont;
	int textTexSize = 128;
#ifdef EKA2
	const CFont *textFont = AknLayoutUtils::FontFromId(EAknLogicalFontSecondaryFont);
	const CFont *numFont = NumberPlain5();
	int maxDim = Rect().Width();
	if (Rect().Height() > maxDim)
		maxDim = Rect().Height();
	if (maxDim >= 320)
		numFont = LatinBold13();
	if (maxDim >= 416) {
		numFont = LatinBold17();
		textTexSize = 256;
	}
#else
	const CFont *textFont = LatinBold13();
	const CFont *numFont = NumberPlain5();
#endif
	if (iDrawMode != EEGLWindow)
		iGLTextFont = makeSymbianGLFont(iGc, Rect().Height());
	else
		iGLTextFont = makeGLFont((const char*) textFont, 0, textTexSize, textTexSize, ' ', '', 'z'+1, ''-1);
	iGLNumFont = makeGLFont((const char*) numFont, 0, 64, 64, '0', '9');
	static_cast<TexGLFont*>(iGLNumFont)->setAllowScaling(false);
	iBantumiGL->setNumFont(iGLNumFont);
	iBantumiGL->setTextFont(iGLTextFont);
}

void CBantumiAppView::SizeChanged() {
	if (iBantumiGL)
		iBantumiGL->reinitGL(Rect().Width(), Rect().Height());

	CGLAppView::SizeChanged();

	if (iBantumiGL)
		InitFonts();
}

void CBantumiAppView::ReinitApp() {
	if (iBantumiGL)
		iBantumiGL->reinitGL(Rect().Width(), Rect().Height());
}

void CBantumiAppView::DeinitGC() {
	if (iFont && iGc)
		iGc->DiscardFont();
}

void CBantumiAppView::InitGC() {
//	iFont = LatinBold13();
	iFont = LatinPlain12();
//	iFont = NumberPlain5();
}

CBantumiAppView::CBantumiAppView() {
}


CBantumiAppView::~CBantumiAppView() {
	DeinitGC();
	delete iBantumi;
	delete iBantumiGL;
	delete iGLNumFont;
	delete iGLTextFont;

	CloseSTDLIB();
}


void CBantumiAppView::UpdateAndDraw() {

	if (iBantumiGL->exit()) {
		static_cast<CBantumiAppUi*>(iEikonEnv->AppUi())->DoExit();
	}

	iBantumi->update();

	iBantumiGL->draw(iTime);

	FinishGLDrawing();

/*	if (iDrawMode != EEGLWindow) {
		eglWaitGL();
		TPoint point(2, 2 + iFont->HeightInPixels());
		TBuf<20> buf;
		buf.Format(_L("%d fps"), iLastFPS);
		iGc->UseFont(iFont);
		iGc->SetPenColor(KRgbWhite);
		iGc->SetDrawMode(CGraphicsContext::EDrawModeXOR);
		iGc->DrawText(buf, point);
		TInt ticklength = iRelTick - iFrameTick[iFrame % iFrameTickLength];
		buf.Format(_L("%d ticks/%d frames"), ticklength, iFrameTickLength);
		point.iX = Rect().Width() - 2 - iFont->TextWidthInPixels(buf);
		iGc->DrawText(buf, point);
		eglWaitNative(EGL_CORE_NATIVE_ENGINE);
	}
*/
}

void CBantumiAppView::FPSUpdated() {
	RFileLogger::WriteFormat(_L("debug"), _L("bantumi.txt"), EFileLoggingModeAppend, _L("%d fps"), iLastFPS);
}

TKeyResponse CBantumiAppView::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType) {
	if (aType != EEventKeyDown)
		return EKeyWasNotConsumed;

	if (iBantumi) {
		switch (aKeyEvent.iScanCode) {
		case EStdKeyDevice3:
			iBantumi->pressed(SELECT);
			break;
		case EStdKeyUpArrow:
			iBantumi->pressed(UP);
			break;
		case EStdKeyDownArrow:
			iBantumi->pressed(DOWN);
			break;
		case EStdKeyLeftArrow:
			iBantumi->pressed(LEFT);
			break;
		case EStdKeyRightArrow:
			iBantumi->pressed(RIGHT);
			break;
		default:
			return EKeyWasNotConsumed;
		}
		return EKeyWasConsumed;
	}
	return EKeyWasNotConsumed;
}

